@@ -58,7 +58,10 @@ lsoda <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   ModelInit <- NULL
   Eventfunc <- NULL
   events <- checkevents(events, times, Ynames, dllname) 
-  
+
+  if (jt == 4 && banddown>0)
+    erow<-matrix(data=0, ncol=n, nrow=banddown) else erow<-NULL
+
   if (is.character(func)) {   # function specified in a DLL
     DLL <- checkDLL(func,jacfunc,dllname,
                     initfunc,verbose,nout, outnames)
@@ -97,7 +100,7 @@ lsoda <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
          
        JacFunc <- function(time,state) {
          attr(state,"names") <- Ynames
-         jacfunc(time,state,parms,...)
+         rbind(jacfunc(time,state,parms,...),erow)
        }
        if (! is.null(events$Type))
          if (events$Type == 2)
@@ -114,7 +117,7 @@ lsoda <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
          func   (time,state,parms,...)
          
        JacFunc <- function(time,state)
-         jacfunc(time,state,parms,...)
+         rbind(jacfunc(time,state,parms,...),erow)
 
        if (! is.null(events$Type))
           if (events$Type == 2)
@@ -136,7 +139,7 @@ lsoda <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     if (!is.matrix(tmp))
       stop("Jacobian function, 'jacfunc' must return a matrix\n")
     dd <- dim(tmp)
-    if((jt ==4 && dd != c(bandup+banddown+1,n)) ||
+    if((jt ==4 && dd != c(bandup+banddown+banddown+1,n)) ||
        (jt ==1 && dd != c(n,n)))
       stop("Jacobian dimension not ok")
     }
