@@ -75,6 +75,10 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   Eventfunc <- NULL
   events <- checkevents(events, times, Ynames, dllname,TRUE)
 
+  ## if (miter == 4) Jacobian should have banddown empty rows
+  if (miter == 4 && banddown>0)
+    erow<-matrix(data=0, ncol=n, nrow=banddown) else erow<-NULL
+
   if (is.character(func)) {   # function specified in a DLL
     DLL <- checkDLL(func,jacfunc,dllname,
                     initfunc,verbose,nout, outnames)
@@ -105,6 +109,7 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     if (is.null(ipar)) ipar<-0
     if (is.null(rpar)) rpar<-0
     Eventfunc <- events$func
+
   } else {
 
     if (is.null(initfunc))
@@ -127,7 +132,7 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
 
       JacFunc <- function(time,state) {
         attr(state,"names") <- Ynames
-        jacfunc(time,state,parms,...)
+        rbind(jacfunc(time,state,parms,...),erow)
       }
       RootFunc <- function(time,state) {
         attr(state,"names") <- Ynames
@@ -147,8 +152,8 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
         func   (time,state,parms,...)
 
       JacFunc <- function(time,state)
-        jacfunc(time,state,parms,...)
-
+        rbind(jacfunc(time,state,parms,...),erow)
+         
       RootFunc <- function(time,state)
         rootfunc(time,state,parms,...)
 
@@ -182,8 +187,8 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
       if (!is.matrix(tmp))
          stop("Jacobian function 'jacfunc' must return a matrix\n")
       dd <- dim(tmp)
-      if ((miter ==4 && dd != c(bandup+banddown+1,n)) ||
-          (miter ==1 && dd != c(n,n)))
+      if ((miter == 4 && dd != c(bandup+banddown+banddown+1,n)) ||
+          (miter == 1 && dd != c(n,n)))
          stop("Jacobian dimension not ok")
      }
   }
@@ -191,10 +196,10 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
 
 ### work arrays iwork, rwork
   # length of rwork and iwork
-  lrw = 20+n*(maxord+1)+3*n  +3*nroot
-  if(miter %in% c(1,2) ) lrw = lrw + 2*n*n+2
-  if(miter ==3)          lrw = lrw + n+2
-  if(miter %in% c(4,5) ) lrw = lrw + (2*banddown+ bandup+1)*n+2
+  lrw <- 20+n*(maxord+1)+3*n  +3*nroot
+  if(miter %in% c(1,2) ) lrw <- lrw + 2*n*n+2
+  if(miter ==3)          lrw <- lrw + n+2
+  if(miter %in% c(4,5) ) lrw <- lrw + (2*banddown+ bandup+1)*n+2
 
   liw   <- if (miter %in% c(0,3)) 20 else 20+n
 
