@@ -11,7 +11,7 @@ SEXP call_rk4(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
 	      SEXP Rpar, SEXP Ipar, SEXP Flist) {
 
   /*  Initialization */
-  init_N_Protect();
+  long int old_N_Protect = save_N_Protected();
 
   double *tt = NULL, *xs = NULL;
   double *tmp, *FF, *out;
@@ -43,6 +43,12 @@ SEXP call_rk4(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
   int nout    = INTEGER(Nout)[0]; /* n of global outputs if func is in a DLL */
   int verbose = INTEGER(Verbose)[0];
 
+  /*------------------------------------------------------------------------*/
+  /* timesteps (for compatibility with lsoda)                               */
+  /*------------------------------------------------------------------------*/
+  timesteps = (double *)R_alloc(2, sizeof(double)); 
+  for (i = 0; i < 2; i++) timesteps[i] = 1;
+
   /*------------------------------------------------------------------------*/
   /* DLL, ipar, rpar (for compatibility with lsoda)                         */
   /*------------------------------------------------------------------------*/
@@ -138,6 +144,10 @@ SEXP call_rk4(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
   for (it = 0; it < nt - 1; it++) {
     t = tt[it];
     dt = tt[it + 1] - t;
+
+    timesteps[0] = timesteps[1];     // experimental, check this
+    timesteps[1] = dt;               // experimental, check this  
+  
     if (verbose)
       Rprintf("Time steps = %d / %d time = %e\n", it + 1, nt, t);
     derivs(Func, t, y0, Parms, Rho, f1, out, 0, neq, ipar, isDll, isForcing);
@@ -185,9 +195,9 @@ SEXP call_rk4(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
     }
   }
   /* Attach essential internal information (codes are compatible to lsoda) */
-  setIstate(R_yout, R_istate, istate, it, 4, 0, 4);
+  setIstate(R_yout, R_istate, istate, it, 4, 0, 4, 0);
 
-  /* Release R Resources */
-  unprotect_all();
+  /* release R resources */
+  restore_N_Protected(old_N_Protect);
   return(R_yout);
 }
